package swami.db;

import java.io.ByteArrayOutputStream;

import org.basex.core.BaseXException;
import org.basex.core.Context;
import org.basex.core.proc.CreateDB;
import org.basex.core.proc.XQuery;

public class ConcurrencyTest {

	private static Context context1;

	static String query1 = "<Give a read query here !!!>";

	static String query2 = "<Give another read query here !!!>";

	static String updateQuery1 = "<Give a write query here !!!>";

	static String updateQuery2 = "<Give another write query here !!!>";

	public static void main(final String[] args) throws Exception {
		context1 = new Context();
		new CreateDB("<give path here>/example.xml",	"example").execute(context1);
		new Thread() {
			public void run() {
				try {
					testConcurrentReadQueries();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}.start(); 
		new Thread() {
			public void run() {
				try {
					testConcurrentWriteQueries();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}.start();
		
	}

	private static void testConcurrentReadQueries() throws Exception {
		Thread t1 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 20; i++) {
						testXquery(query1, context1);
						System.out.println("In thread 1");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		Thread t2 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 20; i++) {
						testXquery(query2, context1);
						System.out.println("In thread 2");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		Thread t3 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 20; i++) {
						testXquery(query2, context1);
						System.out.println("In thread 3");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		t1.start();
		t2.start();
		t3.start();
	}

	private static void testConcurrentWriteQueries() throws Exception {
		Thread t1 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 200; i++) {
						synchronized (ConcurrencyTest.class) {
							testXquery(updateQuery1, context1);
						}
						System.out.println("In thread 1");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		Thread t2 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 200; i++) {
						synchronized (ConcurrencyTest.class) {
							testXquery(updateQuery2, context1);
						}
						System.out.println("In thread 2");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		Thread t3 = new Thread() {
			public void run() {
				try {
					for (int i = 0; i < 200; i++) {
						synchronized (ConcurrencyTest.class) {
							testXquery(updateQuery1, context1);
						}
						System.out.println("In thread 3");
						if (i % 9 == 0) {
							Thread.yield();
						}
					}
				} catch (BaseXException e) {
					e.printStackTrace();
				}
			}
		};
		t1.start();
		t2.start();
		t3.start();
	}

	private static void testXquery(String query, Context context)
			throws BaseXException {
		ByteArrayOutputStream baos;
		baos = new ByteArrayOutputStream();
		new XQuery(query).execute(context, baos);
		System.out.println("result = " + baos.toString());

	}

}
